aboutsummaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
authorDavid Robillard <d@drobilla.net>2016-03-15 23:21:34 -0400
committerDavid Robillard <d@drobilla.net>2019-04-13 19:15:32 +0200
commita549e968cd3f840dc98b2e199c5f52e9f5a5780c (patch)
tree47b9f12595dc62af719a58ccf2e83d20743b0c95
parent499e27a6849cb98235bf9425f1fdb96c8da2c8c5 (diff)
downloadserd-a549e968cd3f840dc98b2e199c5f52e9f5a5780c.tar.gz
serd-a549e968cd3f840dc98b2e199c5f52e9f5a5780c.tar.bz2
serd-a549e968cd3f840dc98b2e199c5f52e9f5a5780c.zip
Use SerdBuffer for mutable buffers
This avoids const violations from abusing SerdChunk as a mutable buffer for string sinks.
-rw-r--r--NEWS6
-rw-r--r--serd/serd.h20
-rw-r--r--src/node.c17
-rw-r--r--src/uri.c10
-rw-r--r--src/writer.c14
-rw-r--r--tests/serd_test.c8
-rw-r--r--wscript4
7 files changed, 47 insertions, 32 deletions
diff --git a/NEWS b/NEWS
index eaa05124..91d99ec9 100644
--- a/NEWS
+++ b/NEWS
@@ -1,3 +1,9 @@
+serd (1.0.0) unstable;
+
+ * Add SerdBuffer for mutable buffers to keep SerdChunk const-correct
+
+ -- David Robillard <d@drobilla.net> Sat, 19 Jan 2019 13:31:12 +0100
+
serd (0.30.1) unstable;
* Fix GCC 4 build
diff --git a/serd/serd.h b/serd/serd.h
index b88cb8f8..a24f3a36 100644
--- a/serd/serd.h
+++ b/serd/serd.h
@@ -230,6 +230,14 @@ typedef struct {
} SerdChunk;
/**
+ A mutable buffer in memory.
+*/
+typedef struct {
+ void* buf; /**< Buffer */
+ size_t len; /**< Size of buffer in bytes */
+} SerdBuffer;
+
+/**
An error description.
*/
typedef struct {
@@ -971,24 +979,24 @@ serd_file_sink(const void* buf, size_t len, void* stream);
/**
A convenience sink function for writing to a string.
- This function can be used as a SerdSink to write to a SerdChunk which is
+ This function can be used as a SerdSink to write to a SerdBuffer which is
resized as necessary with realloc(). The `stream` parameter must point to
- an initialized SerdChunk. When the write is finished, the string should be
- retrieved with serd_chunk_sink_finish().
+ an initialized SerdBuffer. When the write is finished, the string should be
+ retrieved with serd_buffer_sink_finish().
*/
SERD_API
size_t
-serd_chunk_sink(const void* buf, size_t len, void* stream);
+serd_buffer_sink(const void* buf, size_t len, void* stream);
/**
- Finish a serialisation to a chunk with serd_chunk_sink().
+ Finish a serialisation to a chunk with serd_buffer_sink().
The returned string is the result of the serialisation, which is NULL
terminated (by this function) and owned by the caller.
*/
SERD_API
uint8_t*
-serd_chunk_sink_finish(SerdChunk* stream);
+serd_buffer_sink_finish(SerdBuffer* stream);
/**
Set a function to be called when errors occur during writing.
diff --git a/src/node.c b/src/node.c
index 1a7f2fbf..5e903886 100644
--- a/src/node.c
+++ b/src/node.c
@@ -175,27 +175,28 @@ serd_node_new_file_uri(const uint8_t* path,
evil ? "/" : "");
}
- SerdChunk chunk = { uri, uri_len };
+ SerdBuffer buffer = { uri, uri_len };
for (size_t i = 0; i < path_len; ++i) {
if (evil && path[i] == '\\') {
- serd_chunk_sink("/", 1, &chunk);
+ serd_buffer_sink("/", 1, &buffer);
} else if (path[i] == '%') {
- serd_chunk_sink("%%", 2, &chunk);
+ serd_buffer_sink("%%", 2, &buffer);
} else if (!escape || is_uri_path_char(path[i])) {
- serd_chunk_sink(path + i, 1, &chunk);
+ serd_buffer_sink(path + i, 1, &buffer);
} else {
char escape_str[4] = { '%', 0, 0, 0 };
snprintf(escape_str + 1, sizeof(escape_str) - 1, "%X", path[i]);
- serd_chunk_sink(escape_str, 3, &chunk);
+ serd_buffer_sink(escape_str, 3, &buffer);
}
}
- serd_chunk_sink_finish(&chunk);
+ serd_buffer_sink_finish(&buffer);
if (out) {
- serd_uri_parse(chunk.buf, out);
+ serd_uri_parse((const uint8_t*)buffer.buf, out);
}
- return serd_node_from_substring(SERD_URI, chunk.buf, chunk.len);
+ return serd_node_from_substring(
+ SERD_URI, (const uint8_t*)buffer.buf, buffer.len);
}
SerdNode
diff --git a/src/uri.c b/src/uri.c
index f347f010..1508135c 100644
--- a/src/uri.c
+++ b/src/uri.c
@@ -68,25 +68,25 @@ serd_file_uri_parse(const uint8_t* uri, uint8_t** hostname)
++path;
}
- SerdChunk chunk = { NULL, 0 };
+ SerdBuffer buffer = { NULL, 0 };
for (const uint8_t* s = path; *s; ++s) {
if (*s == '%') {
if (*(s + 1) == '%') {
- serd_chunk_sink("%", 1, &chunk);
+ serd_buffer_sink("%", 1, &buffer);
++s;
} else if (is_hexdig(*(s + 1)) && is_hexdig(*(s + 2))) {
const uint8_t code[3] = {*(s + 1), *(s + 2), 0};
const uint8_t c = (uint8_t)strtoul((const char*)code, NULL, 16);
- serd_chunk_sink(&c, 1, &chunk);
+ serd_buffer_sink(&c, 1, &buffer);
s += 2;
} else {
s += 2; // Junk escape, ignore
}
} else {
- serd_chunk_sink(s, 1, &chunk);
+ serd_buffer_sink(s, 1, &buffer);
}
}
- return serd_chunk_sink_finish(&chunk);
+ return serd_buffer_sink_finish(&buffer);
}
bool
diff --git a/src/writer.c b/src/writer.c
index 8dddf51c..e9f928b8 100644
--- a/src/writer.c
+++ b/src/writer.c
@@ -976,18 +976,18 @@ serd_file_sink(const void* buf, size_t len, void* stream)
}
size_t
-serd_chunk_sink(const void* buf, size_t len, void* stream)
+serd_buffer_sink(const void* buf, size_t len, void* stream)
{
- SerdChunk* chunk = (SerdChunk*)stream;
- chunk->buf = (uint8_t*)realloc((uint8_t*)chunk->buf, chunk->len + len);
- memcpy((uint8_t*)chunk->buf + chunk->len, buf, len);
- chunk->len += len;
+ SerdBuffer* buffer = (SerdBuffer*)stream;
+ buffer->buf = (char*)realloc(buffer->buf, buffer->len + len);
+ memcpy((uint8_t*)buffer->buf + buffer->len, buf, len);
+ buffer->len += len;
return len;
}
uint8_t*
-serd_chunk_sink_finish(SerdChunk* stream)
+serd_buffer_sink_finish(SerdBuffer* stream)
{
- serd_chunk_sink("", 1, stream);
+ serd_buffer_sink("", 1, stream);
return (uint8_t*)stream->buf;
}
diff --git a/wscript b/wscript
index 68584414..f3dd30eb 100644
--- a/wscript
+++ b/wscript
@@ -11,8 +11,8 @@ from waflib.extras import autowaf
# major increment <=> incompatible changes
# minor increment <=> compatible changes (additions)
# micro increment <=> no interface changes
-SERD_VERSION = '0.30.1'
-SERD_MAJOR_VERSION = '0'
+SERD_VERSION = '1.0.0'
+SERD_MAJOR_VERSION = '1'
# Mandatory waf variables
APPNAME = 'serd' # Package name for waf dist
diff --git a/tests/serd_test.c b/tests/serd_test.c
index e537a5f5..12899de2 100644
--- a/tests/serd_test.c
+++ b/tests/serd_test.c
@@ -484,16 +484,16 @@ main(void)
serd_writer_free(writer);
- // Test chunk sink
- SerdChunk chunk = { NULL, 0 };
+ // Test buffer sink
+ SerdBuffer buffer = { NULL, 0 };
writer = serd_writer_new(
- SERD_TURTLE, (SerdStyle)0, env, NULL, serd_chunk_sink, &chunk);
+ SERD_TURTLE, (SerdStyle)0, env, NULL, serd_buffer_sink, &buffer);
o = serd_node_from_string(SERD_URI, USTR("http://example.org/base"));
assert(!serd_writer_set_base_uri(writer, &o));
serd_writer_free(writer);
- uint8_t* out = serd_chunk_sink_finish(&chunk);
+ uint8_t* out = serd_buffer_sink_finish(&buffer);
assert(!strcmp((const char*)out, "@base <http://example.org/base> .\n"));
serd_free(out);